#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
using System.Collections.Generic;
using System.Linq;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "SortParticles", Category = "Value")]
	#endregion PluginInfo
	public class ValueSortParticlesNode : IPluginEvaluate
	{
		#region fields & pins
	
		[Input("LayerOrderIndices")]
		public ISpread<UInt32> LayerOrderIndices;

		[Input("Positions")]
		public ISpread<Vector3D> Positions;

		[Input("AlivePointers")]
		public ISpread<UInt32> AlivePointers;

		[Input("ParticleIndices")]
		public ISpread<UInt32> ParticleIndices;

		[Input("MaxAlive")]
		public ISpread<UInt32> MaxAlive;

		[Output("Output")]
		public ISpread<Vector3D> FOutput;
		
		[Import()]
		public ILogger FLogger;
		#endregion fields & pins		

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			FOutput.SliceCount = Convert.ToInt32(MaxAlive[0]);
			List<Particle> particles = new List<Particle>();
			
			// trying to access only alive particles. This doesn't work yet.			
			for (int c = 0; c < MaxAlive[0]; c++) {
				int AlivePointer = Convert.ToInt32(AlivePointers[c]);
				int ParticleIndex = Convert.ToInt32(ParticleIndices[AlivePointer]);
				particles.Add(new Particle(Convert.ToInt32(LayerOrderIndices[ParticleIndex]), Positions[ParticleIndex]));
			}
			
			// this basically works but uses all particles. 			
//			for (int c = 0; c < SpreadMax; c++) {
//				particles.Add(new Particle(Convert.ToInt32(LayerOrderIndices[c]), Positions[c]));
//			}
			
			
			particles = particles.OrderBy(particle => particle.LayerOrderIndices).ToList();
			for (int i = 0; i < particles.Count(); i++)
				FOutput[i] = particles[i].position;
			
			//FLogger.Log(LogType.Debug, "hi tty!");
		}
	}
}


 class Particle
{
    public Vector3D position;
    public int LayerOrderIndices;

	public Particle(int LayerOrderIndices, Vector3D position) {
        this.position = position;
        this.LayerOrderIndices = LayerOrderIndices;
    }
	
}